# -*- coding: utf-8 -*-
# cython: language_level = 3


import pprint
from ast import literal_eval
from typing import override

from .._io_protocol import SupportsReadAndReadline
from .._io_protocol import SupportsWrite
from ..abc import ABCConfigFile
from ..base import ConfigData
from ..errors import FailedProcessConfigFileError
from ..main import BaseLocalFileConfigSL


class PythonLiteralSL(BaseLocalFileConfigSL):
    """
    Python 字面量序列化处理器
    """

    @property
    @override
    def processor_reg_name(self) -> str:
        return "python_literal"

    @property
    @override
    def file_ext(self) -> tuple[str, ...]:
        return ".py", ".python_literal", ".pyl"

    @override
    def save_file(
            self,
            config_file: ABCConfigFile,
            target_file: SupportsWrite[str],
            *merged_args,
            **merged_kwargs
    ) -> None:
        try:
            target_file.write(pprint.pformat(config_file.data, *merged_args, **merged_kwargs))
        except Exception as e:
            raise FailedProcessConfigFileError(e) from e

    @override
    def load_file[C: ABCConfigFile](
            self, config_file_cls: type[C],
            source_file: SupportsReadAndReadline[str],
            *merged_args,
            **merged_kwargs
    ) -> C:
        try:
            data = literal_eval(source_file.read())
        except Exception as e:
            raise FailedProcessConfigFileError(e) from e

        return config_file_cls(ConfigData(data), config_format=self.processor_reg_name)


__all__ = (
    "PythonLiteralSL",
)
